# mafico: lightweight map, filter, collect

mafico is meant to be a very minimalistic and lightweight
map/filter/collect library.

## **ma**p

```c++
    // std::vector<int> -> std::vector<int>
    std::vector primes{2, 3, 5, 7, 11};
    auto prime_sqr = mafico::map(primes, [](auto p){ return p*p; });

    // std::vector<Person> -> std::vector<const char*>
    struct Person {
        const char* name = "no one";
        int age = 0;
    };

    std::vector<Person> people{{"Victor", 25}, {"Romario", 31}, {"Praggnanandhaa", 15}};
    auto names = mafico::map(people, &Person::name);

    // std::unordered_map<const char*, int> -> std::vector<const char*>
    std::unordered_map<const char*, int> people_map{{"Victor", 25}, {"Romario", 31}, {"Praggnanandhaa", 15}};
    auto map_names = mafico::map_to<std::vector>(people_map, [](auto& e){ return e.first });
```

## **fi**lter

```c++
    std::vector<double> numbers{1.0, 1.1, 2.0, 3.5, 3.8, 4.0, 5.0, 5.5};
    auto integers = mafico::filter(numbers, is_integer); // integers is std::vector<double>
    auto int_integers = mafico::filter_to<int>(numbers, is_integer); // integers is std::vector<int>
```

# **co**llect

```c++
    std::vector<int> primes{2, 3, 5, 7, 11};
    auto sum = mafico::collect(primes, [](auto& total, auto& p){ return total + p; });
```
