#ifndef MAFICO_FILTER
#define MAFICO_FILTER

#include <algorithm>
#include <functional>

namespace mafico
{

// filter(Container<T>, T -> bool) -> Container<T>
template <
    template <typename> typename Container,
    typename T,
    typename FilterF,
    typename U = T>
[[nodiscard]] auto filter(const Container<T>& c, FilterF&& f)
{
    Container<U> result;

    std::copy_if(
        c.begin(), c.end(), std::back_inserter(result),
        [f = std::forward<FilterF>(f)](auto&& e) mutable
        {
            return std::invoke(std::forward<FilterF>(f), e);
        });

    return result;
}

template <
    typename U,
    template <typename> typename Container,
    typename T,
    typename FilterF>
[[nodiscard]] auto filter_to(const Container<T>& c, FilterF&& f)
{
    return filter<Container, T, FilterF, U>(c, std::forward<FilterF>(f));
}

}

#endif
