#ifndef MAFICO_MAP
#define MAFICO_MAP

#include <algorithm>
#include <functional>
#include <type_traits>

namespace mafico
{

namespace detail {
    template <typename InContainer, typename OutContainer, typename MapF>
    [[nodiscard]] auto map_impl(const InContainer& c, MapF&& f)
    {
        OutContainer result;

        std::transform(
            c.begin(), c.end(), std::back_inserter(result),
            [f = std::forward<MapF>(f)](auto&& e) mutable
            {
                return std::invoke(std::forward<MapF>(f), e);
            });

        return result;
    }

    template <typename Container, typename MapF>
    using U = std::remove_reference_t<
        std::invoke_result_t<MapF, typename Container::value_type&>>;
}

// map(Container<T>, T -> U) -> Container<U>
template <
    template <typename> typename Container,
    typename MapF,
    typename T>
[[nodiscard]] auto map(const Container<T>& c, MapF&& f)
{
    using U = detail::U<Container<T>, MapF>;

    return detail::map_impl<Container<T>, Container<U>>(
        c, std::forward<MapF>(f));
}

// map<OutContainer>(InContainer<T>, T -> U) -> OutContainer<U>
template <
    template <typename> typename OutContainer,
    typename InContainer,
    typename MapF>
[[nodiscard]] auto map_to(const InContainer& c, MapF&& f)
{
    using U = detail::U<InContainer, MapF>;

    return detail::map_impl<InContainer, OutContainer<U>>(
        c, std::forward<MapF>(f));
}

}

#endif
