#include <mafico/map.hpp>

#include <list>
#include <vector>
#include <unordered_map>

#include "test_common.hpp"

int main()
{
    std::list<double> pow2{1, 2, 4, 8, 16, 32, 64};
    auto inverse = mafico::map(pow2, [](auto p){ return 1/p; });

    test_var_type<std::list<double>>(inverse);
    test_cond((inverse == std::list{1./1, 1./2, 1./4, 1./8, 1./16, 1./32, 1./64}));

    std::vector<float> primes{2, 3, 5, 7, 11, 13, 17};
    auto primesqr = mafico::map(primes, [](auto& p){ return p * p; });

    test_var_type<std::vector<float>>(primesqr);
    test_cond((primesqr == std::vector{4.f, 9.f, 25.f, 49.f, 121.f, 169.f, 289.f}));

    struct Person
    {
        const char* name = "No one";
        unsigned age = 0;
    };

    std::vector<Person> people{
        {"Victor", 25},
        {"Romario", 31},
        {"Praggnanandhaa", 15}
    };

    auto names = mafico::map(people, &Person::name);

    test_var_type<std::vector<const char*>>(names);
    test_cond((names == std::vector{"Victor", "Romario", "Praggnanandhaa"}));

    std::unordered_map<const char*, int> people_map{
        {"Victor", 25},
        {"Romario", 31},
        {"Praggnanandhaa", 15}
    };

    auto map_names = mafico::map_to<std::vector>(
        people_map, [](auto& e){ return e.first; });

    test_var_type<std::vector<const char*>>(map_names);
    test_cond((std::is_permutation(
                   names.begin(), names.end(), map_names.begin())));

    return 0;
}
